// *******************************************************************
// GraphWin.h
// Ver. 1.00
// Component for C++ Builder
// Copyright (c) 1998 by Roy X. Hong
// hong@hpl.crestech.ca,  Toronto, Canada
// *******************************************************************

// Ver. 3.01  Dec. 1988, for C++ Builder3

// Ver. 1.01, Mar. 31, 1998
// Fixed a few bugs.
// Added Clipping for Graph and CopyToClipBoard function


//---------------------------------------------------------------------------
#ifndef GraphWinH
#define GraphWinH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Dsgnintf.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\ExtCtrls.hpp>
#include <vcl\printers.hpp>
#include <vcl\Buttons.hpp>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define Real    float
#define EMPTY   9999999999.9
#define MaxNumOfYSeries    10
#define MaxR    9999999999.9
#define MinR    -9999999999.9

#define Round(x)    ((int)((x)+0.5))

enum AXIS_TYPE { Linear, Xlog, Ylog, XlogYlog };
enum BKGD_TYPE { Normal, Gradient, Bitmap };
enum DRAW_TYPE { OnScreen, Preview, PrintOut };
enum DOTSYMBOL_TYPE { SolidSquare, EmptySquare, TriAngle, RevTriAngle,
        SolidCircle, EmptyCircle, Cross, TiltCross, VertRect, HoriRect,
        SolidDiamond, EmptyDiamond };
enum GRAPH_TYPE { Line, Dot, Bar, Bezier, BSpline, CubSpline, Regression };

enum XEDIT_ORDER { XGT, XTS, XSD, XLS, XLC, XDS, XLW, XLG, XBW,
                   XBF, XBY, XEB, XAA };

// *******************************************************************
// TPlotScale.cpp
// *******************************************************************

class PACKAGE TPlotScale : public TGraphicControl
{
protected:
    Real xl, xr, yt, yb, xmin, ymin, xd, yd;
public:
    __fastcall TPlotScale(TComponent* Owner) : TGraphicControl(Owner) {}
    __fastcall ~TPlotScale() {}
    void __fastcall NewRec(int xx1, int yy1, int xx2, int yy2);
    void __fastcall SetExt(Real xx1, Real yy1, Real xx2, Real yy2);
    int __fastcall xx(Real x);
    int __fastcall yyr(Real y);
#ifdef __MyOwnVersion__
    void __fastcall NewRec(TRect rect);
    void __fastcall SetExt(TRect rect);
    int __fastcall yy(Real y);
    Real __fastcall _xx(int x);
    Real __fastcall _yy(int y);
    Real __fastcall _yyr(int y);
#endif
};

// *******************************************************************
// TAxisWin.cpp
// *******************************************************************

class PACKAGE TAxis : public TPersistent
{
private:
    TNotifyEvent FOnChange;
    void __fastcall Changed(void);
public:
    TColor FAxisColor, FLabelColor, FGridColor, FBkgdColor, FSubTitleColor;
    AXIS_TYPE FAxisType;
    BKGD_TYPE FBkgdType;
    AnsiString FMainTitle, FSubTitle, FXTitle, FYTitle;
    TFont *FMainTitleFont;
    bool FShowVGrid, FShowHGrid, FLegendAtRight, FShowLegend;
    Graphics::TBitmap *FBmp;
    int ShortTickLength, LongTickLength, ZeroXFlag;
    Real FXmin, FYmin, FXmax, FYmax, FXstep, FYstep;
    bool FXminAuto, FYminAuto, FXmaxAuto, FYmaxAuto, FXstepAuto, FYstepAuto;

    __fastcall TAxis(void);
    __fastcall virtual ~TAxis(void);
    void __fastcall SetAxisColor(TColor v);
    void __fastcall SetLabelColor(TColor v);
    void __fastcall SetGridColor(TColor v);
    void __fastcall SetBkgdColor(TColor v);
    void __fastcall SetAxisType(AXIS_TYPE v);
    void __fastcall SetBkgdType(BKGD_TYPE v);
    void __fastcall SetShowHGrid(bool v);
    void __fastcall SetShowVGrid(bool v);
    void __fastcall SetShortTickLength(int v);
    void __fastcall SetLongTickLength(int v);

    void __fastcall SetXTitle(AnsiString s);
    void __fastcall SetYTitle(AnsiString s);
    void __fastcall SetMainTitle(AnsiString s);
    void __fastcall SetSubTitle(AnsiString s);
    void __fastcall SetSubTitleColor(TColor v);
    void __fastcall SetShowLegend(bool v);
    void __fastcall SetLegendAtRight(bool v);
    void __fastcall SetMainTitleFont(TFont *v);
    void __fastcall SetBitmap(Graphics::TBitmap *v);
    void __fastcall LoadBitmap(char *filename);

    void __fastcall SetXMin(Real v);
    void __fastcall SetYMin(Real v);
    void __fastcall SetXMax(Real v);
    void __fastcall SetYMax(Real v);
    void __fastcall SetXStep(Real v);
    void __fastcall SetYStep(Real v);
    void __fastcall SetXMinAuto(bool v);
    void __fastcall SetYMinAuto(bool v);
    void __fastcall SetXMaxAuto(bool v);
    void __fastcall SetYMaxAuto(bool v);
    void __fastcall SetXStepAuto(bool v);
    void __fastcall SetYStepAuto(bool v);
__published:
    __property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
    __property TColor AxisColor = {read=FAxisColor, write=SetAxisColor, default=clWhite};
    __property TColor LabelColor = {read=FLabelColor, write=SetLabelColor, default=clWhite};
    __property TColor GridColor = {read=FGridColor, write=SetGridColor, default=clGray};
    __property TColor BkgdColor = {read=FBkgdColor, write=SetBkgdColor, default=clBlack};
    __property AXIS_TYPE AxisType = {read=FAxisType, write=SetAxisType, default=Linear};
    __property BKGD_TYPE BkgdType = {read=FBkgdType, write=SetBkgdType, default=Normal};
    __property bool ShowHGrid = {read=FShowHGrid, write=SetShowHGrid, default=TRUE};
    __property bool ShowVGrid = {read=FShowVGrid, write=SetShowVGrid, default=TRUE};

    __property AnsiString MainTitle = {read=FMainTitle, write=SetMainTitle};
    __property AnsiString SubTitle = {read=FSubTitle, write=SetSubTitle};
    __property AnsiString XTitle = {read=FXTitle, write=SetXTitle};
    __property AnsiString YTitle = {read=FYTitle, write=SetYTitle};
    __property TColor SubTitleColor = {read=FSubTitleColor, write=SetSubTitleColor, default=clWhite};
    __property TFont* MainTitleFont = {read=FMainTitleFont, write=SetMainTitleFont};
    __property bool ShowLegend = {read=FShowLegend, write=SetShowLegend, default=TRUE};
    __property bool LegendAtRight = {read=FLegendAtRight, write=SetLegendAtRight, default=TRUE};
    __property Graphics::TBitmap *Bitmap = {read=FBmp, write=SetBitmap, default=0};

    __property Real Xmin = {read=FXmin, write=SetXMin, default=0};
    __property Real Xmax = {read=FXmax, write=SetXMax};
    __property Real Ymin = {read=FYmin, write=SetYMin};
    __property Real Ymax = {read=FYmax, write=SetYMax};
    __property Real Xstep = {read=FXstep, write=SetXStep};
    __property Real Ystep = {read=FYstep, write=SetYStep};
    __property bool XminAuto = {read=FXminAuto, write=SetXMinAuto, default=true};
    __property bool XmaxAuto = {read=FXmaxAuto, write=SetXMaxAuto};
    __property bool YminAuto = {read=FYminAuto, write=SetYMinAuto};
    __property bool YmaxAuto = {read=FYmaxAuto, write=SetYMaxAuto};
    __property bool XstepAuto = {read=FXstepAuto, write=SetXStepAuto};
    __property bool YstepAuto = {read=FYstepAuto, write=SetYStepAuto};
};


class PACKAGE TAxisWin : public TPlotScale
{
private:
    char str[40];
protected:
    int LeftMargin, TopMargin, RightMargin, BottomMargin;
    int LegendWidth, LegendHeight, TextHeight;
    int XNumberOffset, YNumberOffset;

    Real _xmin, _ymin, _xmax, _ymax, _xstep, _ystep;
    Real efn, xstart, xend, ystart, yend;
    int X1, Y1, X2, Y2, xs, ys, xe, ye, ndig, ndec, power;
    Graphics::TBitmap *MemBmp;
    TCanvas *CanvasPtr, *OldPtr;
    DRAW_TYPE PrintFlag;
    Real PrtRatio;
    int PrtWidth, PrtHeight, PrtXOff, PrtYOff;

    void __fastcall NumInfo(Real nr);
    void __fastcall SelP(void);
    void __fastcall LiMinMax(Real& min, Real& max, Real& inc, bool autoinc);
    void __fastcall LogMinMax(Real& min, Real& max);
    void __fastcall CalMinMax(void);
    int __fastcall CalMaxYNumberWidth(void);
    void __fastcall F2Str(Real v, int n_dig, int n_dec, char *s);
    void __fastcall LiXAxis(void);
    void __fastcall LiYAxis(void);
    void __fastcall LogXAxis(void);
    void __fastcall LogYAxis(void);

    void __fastcall PutMainTitle(void);
    void __fastcall PutSubTitle(void);
    void __fastcall PutXTitle(void);
    void __fastcall VerticalText(int x, int y, char *s);
    void __fastcall PutYTitle(void);
    void __fastcall Clear(void);
    void __fastcall GradientBackground(void);
    void __fastcall Layout(void);
public:
    TAxis *FAxis;
    __fastcall TAxisWin(TComponent* Owner);
    __fastcall ~TAxisWin();
    void __fastcall DrawAxis(void);
    void virtual __fastcall DrawGraph(void) {}
    void __fastcall StyleChanged(TObject* Sender);
__published:
    __property TAxis *Axis = {read=FAxis, write=FAxis, default=0};
    __property Align;
    __property OnClick;
    __property OnDblClick;
    __property OnDragDrop;
    __property OnDragOver;
    __property OnEndDrag;
    __property OnMouseDown;
    __property OnMouseMove;
    __property OnMouseUp;
    __property Font;
};

// *******************************************************************
// TGraphWin.cpp
// *******************************************************************

class PACKAGE TAboutProperty : public TPropertyEditor
{
public:
    __fastcall TAboutProperty() {}
    __fastcall ~TAboutProperty() {}
    void __fastcall Edit();
    TPropertyAttributes __fastcall GetAttributes();
    AnsiString __fastcall GetValue();
};

class __declspec(delphiclass) TGraphWin;

class PACKAGE TPreviewForm : public TForm
{
__published:
    TPanel *Panel1;
    TBitBtn *BitBtn1;
    TBitBtn *BitBtn2;
    TPaintBox *PaintBox1;
    void __fastcall FormPaint(TObject *Sender);
public:
    __fastcall TPreviewForm(TComponent* Owner);
    __fastcall TPreviewForm(TComponent* Owner, Graphics::TBitmap *Bp, TGraphWin *gw);
    __fastcall ~TPreviewForm() {}
    Graphics::TBitmap *BmpPtr;
    TGraphWin *GW;
};


class PACKAGE TPrintPage : public TPersistent
{
public:
    int FTopMargin, FLeftMargin, FRightMargin, FBottomMargin;
    bool TopFlag, LeftFlag, RightFlag, BottomFlag;

    __fastcall TPrintPage(void);
    __fastcall virtual ~TPrintPage(void) {}
    void __fastcall SetTopMargin(int v);
    void __fastcall SetLeftMargin(int v);
    void __fastcall SetRightMargin(int v);
    void __fastcall SetBottomMargin(int v);
__published:
    __property int TopMargin = {read=FTopMargin, write=SetTopMargin, default=-1};
    __property int LeftMargin = {read=FLeftMargin, write=SetLeftMargin, default=-1};
    __property int RightMargin = {read=FRightMargin, write=SetRightMargin, default=-1};
    __property int BottomMargin = {read=FBottomMargin, write=SetBottomMargin, default=-1};
};


class PACKAGE TGraph : public TPersistent
{
private:
    TNotifyEvent FOnChange;
    void __fastcall Changed(void);
    TStringList *FEditAll;

    void __fastcall SaveGraphType();        //XGT
    void __fastcall ReadGraphType();
    void __fastcall SaveShowThisYSeries();  //XTS
    void __fastcall ReadShowThisYSeries();
    void __fastcall SaveShowDotSymbol();    //XDS
    void __fastcall ReadShowDotSymbol();
    void __fastcall SaveShowErrorBar();     //XEB
    void __fastcall ReadShowErrorBar();
    void __fastcall SaveLineStyle();        //XLS
    void __fastcall ReadLineStyle();
    void __fastcall SaveLineColor();        //XLC
    void __fastcall ReadLineColor();
    void __fastcall SaveDotSymbol();        //XDS
    void __fastcall ReadDotSymbol();
    void __fastcall SaveLineWidth();        //XLW
    void __fastcall ReadLineWidth();
    void __fastcall SaveLegendStr();        //XLG
    void __fastcall ReadLegendStr();
    void __fastcall SaveBarWidth();         //XBW
    void __fastcall ReadBarWidth();
    void __fastcall SaveBarShift();         //XBF
    void __fastcall ReadBarShift();
    void __fastcall SaveBarStyle();         //XBY
    void __fastcall ReadBarStyle();
    void __fastcall SaveAntiAliasing();     //XAA
    void __fastcall ReadAntiAliasing();

public:
    GRAPH_TYPE FGraphType[MaxNumOfYSeries];
    bool FShowDotSymbol[MaxNumOfYSeries], FShowThisYSeries[MaxNumOfYSeries];
    bool FShowErrorBar[MaxNumOfYSeries], FAntiAliasing[MaxNumOfYSeries];
    TColor FLineColor[MaxNumOfYSeries];
    int FLineWidth[MaxNumOfYSeries], FCurrentYSeries;
    int FBarWidth[MaxNumOfYSeries], FBarShift[MaxNumOfYSeries];
    TBrushStyle FBarStyle[MaxNumOfYSeries];
    TPenStyle FLineStyle[MaxNumOfYSeries];
    DOTSYMBOL_TYPE FDotSymbol[MaxNumOfYSeries];
    AnsiString FLegendStr[MaxNumOfYSeries];

    __fastcall TGraph(void);
    __fastcall virtual ~TGraph(void) { delete FEditAll; }

    void __fastcall SetGraphType(GRAPH_TYPE v);
    void __fastcall SetShowDotSymbol(bool v);
    void __fastcall SetShowThisYSeries(bool v);
    void __fastcall SetShowErrorBar(bool v);
    void __fastcall SetLineColor(TColor v);
    void __fastcall SetLineWidth(int v);
    void __fastcall SetLineStyle(TPenStyle v);
    void __fastcall SetDotSymbol(DOTSYMBOL_TYPE v);
    void __fastcall SetLegendStr(AnsiString v);
    void __fastcall SetBarWidth(int v);
    void __fastcall SetBarShift(int v);
    void __fastcall SetBarStyle(TBrushStyle v);
    void __fastcall SetAntiAliasing(bool v);

    GRAPH_TYPE __fastcall GetGraphType();
    bool __fastcall GetShowThisYSeries();
    bool __fastcall GetShowDotSymbol();
    bool __fastcall GetShowErrorBar();
    TColor __fastcall GetLineColor();
    int  __fastcall GetLineWidth();
    TPenStyle __fastcall GetLineStyle();
    DOTSYMBOL_TYPE __fastcall GetDotSymbol();
    AnsiString __fastcall GetLegendStr();
    int __fastcall GetBarWidth();
    int __fastcall GetBarShift();
    TBrushStyle __fastcall GetBarStyle();
    bool __fastcall GetAntiAliasing();

    void __fastcall SetCurrentYSeries(int v) { FCurrentYSeries=v; }
    void __fastcall ReadEditAll();
    void __fastcall SetEditAll(TStringList *v);

__published:
    __property TNotifyEvent OnChange = {read=FOnChange, write=FOnChange};
    __property GRAPH_TYPE GraphType = {read=GetGraphType, write=SetGraphType, stored=false};
    __property int CurrentYSeries = {read=FCurrentYSeries, write=FCurrentYSeries, stored=false};
    __property bool ShowThisYSeries = {read=GetShowThisYSeries, write=SetShowThisYSeries, stored=false};
    __property int LineWidth = {read=GetLineWidth, write=SetLineWidth, stored=false};
    __property TColor LineColor = {read=GetLineColor, write=SetLineColor, stored=false};
    __property TPenStyle LineStyle = {read=GetLineStyle, write=SetLineStyle, stored=false};
    __property DOTSYMBOL_TYPE DotSymbol = {read=GetDotSymbol, write=SetDotSymbol, stored=false};
    __property bool ShowDotSymbol = {read=GetShowDotSymbol, write=SetShowDotSymbol, stored=false};
    __property bool ShowErrorBar = {read=GetShowErrorBar, write=SetShowErrorBar, stored=false};
    __property AnsiString Legend = {read=GetLegendStr, write=SetLegendStr, stored=false};
    __property int BarWidth = {read=GetBarWidth, write=SetBarWidth, stored=false};
    __property int BarShift = {read=GetBarShift, write=SetBarShift, stored=false};
    __property TBrushStyle BarStyle = {read=GetBarStyle, write=SetBarStyle, stored=false};
    __property bool AntiAliasing = {read=GetAntiAliasing, write=SetAntiAliasing, stored=false};
    __property TStringList *XEditAll = {read=FEditAll, write=SetEditAll};
};

#define AA_COLOR_LEVELS        32

class PACKAGE TGraphWin : public TAxisWin
{
private:
    TPreviewForm *PreviewForm;
    TAboutProperty *FAbout;
    Real *XData, *YData, *YError, EmptyMark, SamplingFrq, *xt, *yt, *et;
    int FNumOfXPoints, FNumOfYSeries, xp, yp, DrawXN;
    bool XDataFlag;

    int AAxpre, AAypre, AAxd, AAyd, AAxdOld, AAydOld;
    TColor AAColor[AA_COLOR_LEVELS];

    void __fastcall cubic(int n, int *x, Real *y, Real *y2, Real *g, Real *f);
protected:
    void __fastcall Loaded();
    void __fastcall DrawDotSymbol(int v);
    void __fastcall DrawLine(bool anti_alias);
    void __fastcall DrawDot(int yj);
    void __fastcall DrawError(yj);
    void __fastcall DrawBar(int yj);

    void __fastcall SetAAColor(TColor co);
    void __fastcall AALineTo(int X1, int Y1);
    void __fastcall InitAAFilter(int x, int y, TColor co);
    void __fastcall AAFilter(int x, int y);

    void __fastcall DrawBezier(bool anti_alias);
    void __fastcall DrawBSpline(bool anti_alias);
    void __fastcall DrawCubSpline(bool anti_alias);
    void __fastcall DrawRegLine(bool anti_alias);
    void __fastcall StyleChanged(TObject* Sender);
    void __fastcall SetDrawColor(int j);
    void __fastcall DrawChart(int j);
    void __fastcall DrawLegend(int j, int w, int h);
    void __fastcall ScaleData(void);
public:
    TGraph *FGraph;
    TPrintPage *FPrintPage;
    __fastcall TGraphWin(TComponent* Owner);
    __fastcall ~TGraphWin();
    void __fastcall GetXData(Real *x);
    void __fastcall GetYData(Real *y);
    void __fastcall GetYError(Real *e);
    void __fastcall SetNumOfXPoints(int v);
    void __fastcall SetNumOfYSeries(int v);
    void __fastcall FindMinMax(void);
    void __fastcall SetPrintFlag(DRAW_TYPE v) { PrintFlag=v; }
    void __fastcall DrawGraph(void);
    void __fastcall Paint(void);
    void __fastcall Print(void);
    void __fastcall CopyToClipBoard(void);
__published:
    __property TGraph *Graph = {read=FGraph, write=FGraph, default=0};
    __property TPrintPage *PrintPage = {read=FPrintPage, write=FPrintPage, default=0};
    __property int NumOfXPoints = {read=FNumOfXPoints, write=SetNumOfXPoints};
    __property int NumOfYSeries = {read=FNumOfYSeries, write=SetNumOfYSeries};
    __property TAboutProperty *About = {read=FAbout, write=FAbout};
};

//namespace GraphWin
//{
//    extern void __fastcall Register(void);
//}

#endif
